/*
 * This file contains parts of FFmpeg.
 *
 * Copyright (c) 2002 Fabrice Bellard.
 * Copyright (c) 2004 Roman Shaposhnik.
 *
 * FFmpeg is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * FFmpeg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <stdlib.h>
#include <memory.h>
#include "../inttypes.h"

#include "ffmpeg_dv.h"


typedef short DCTELEM;



#define TEX_VLC_BITS 9

static const RL_VLC_ELEM dv_rl_vlc[1184] = {
	{   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, 
	{   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, 
	{   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, 
	{   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, 
	{   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, 
	{   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, 
	{   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, 
	{   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, 
	{   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, 
	{   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, 
	{   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, 
	{   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, 
	{   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, 
	{   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, 
	{   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, 
	{   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, {   16,   3,   1}, 
	{  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, 
	{  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, 
	{  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, 
	{  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, 
	{  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, 
	{  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, 
	{  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, 
	{  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, 
	{  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, 
	{  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, 
	{  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, 
	{  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, 
	{  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, 
	{  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, 
	{  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, 
	{  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, {  -16,   3,   1}, 
	{   32,   4,   1}, {   32,   4,   1}, {   32,   4,   1}, {   32,   4,   1}, 
	{   32,   4,   1}, {   32,   4,   1}, {   32,   4,   1}, {   32,   4,   1}, 
	{   32,   4,   1}, {   32,   4,   1}, {   32,   4,   1}, {   32,   4,   1}, 
	{   32,   4,   1}, {   32,   4,   1}, {   32,   4,   1}, {   32,   4,   1}, 
	{   32,   4,   1}, {   32,   4,   1}, {   32,   4,   1}, {   32,   4,   1}, 
	{   32,   4,   1}, {   32,   4,   1}, {   32,   4,   1}, {   32,   4,   1}, 
	{   32,   4,   1}, {   32,   4,   1}, {   32,   4,   1}, {   32,   4,   1}, 
	{   32,   4,   1}, {   32,   4,   1}, {   32,   4,   1}, {   32,   4,   1}, 
	{  -32,   4,   1}, {  -32,   4,   1}, {  -32,   4,   1}, {  -32,   4,   1}, 
	{  -32,   4,   1}, {  -32,   4,   1}, {  -32,   4,   1}, {  -32,   4,   1}, 
	{  -32,   4,   1}, {  -32,   4,   1}, {  -32,   4,   1}, {  -32,   4,   1}, 
	{  -32,   4,   1}, {  -32,   4,   1}, {  -32,   4,   1}, {  -32,   4,   1}, 
	{  -32,   4,   1}, {  -32,   4,   1}, {  -32,   4,   1}, {  -32,   4,   1}, 
	{  -32,   4,   1}, {  -32,   4,   1}, {  -32,   4,   1}, {  -32,   4,   1}, 
	{  -32,   4,   1}, {  -32,   4,   1}, {  -32,   4,   1}, {  -32,   4,   1}, 
	{  -32,   4,   1}, {  -32,   4,   1}, {  -32,   4,   1}, {  -32,   4,   1}, 
	{    0,   4, 128}, {    0,   4, 128}, {    0,   4, 128}, {    0,   4, 128}, 
	{    0,   4, 128}, {    0,   4, 128}, {    0,   4, 128}, {    0,   4, 128}, 
	{    0,   4, 128}, {    0,   4, 128}, {    0,   4, 128}, {    0,   4, 128}, 
	{    0,   4, 128}, {    0,   4, 128}, {    0,   4, 128}, {    0,   4, 128}, 
	{    0,   4, 128}, {    0,   4, 128}, {    0,   4, 128}, {    0,   4, 128}, 
	{    0,   4, 128}, {    0,   4, 128}, {    0,   4, 128}, {    0,   4, 128}, 
	{    0,   4, 128}, {    0,   4, 128}, {    0,   4, 128}, {    0,   4, 128}, 
	{    0,   4, 128}, {    0,   4, 128}, {    0,   4, 128}, {    0,   4, 128}, 
	{   16,   5,   2}, {   16,   5,   2}, {   16,   5,   2}, {   16,   5,   2}, 
	{   16,   5,   2}, {   16,   5,   2}, {   16,   5,   2}, {   16,   5,   2}, 
	{   16,   5,   2}, {   16,   5,   2}, {   16,   5,   2}, {   16,   5,   2}, 
	{   16,   5,   2}, {   16,   5,   2}, {   16,   5,   2}, {   16,   5,   2}, 
	{  -16,   5,   2}, {  -16,   5,   2}, {  -16,   5,   2}, {  -16,   5,   2}, 
	{  -16,   5,   2}, {  -16,   5,   2}, {  -16,   5,   2}, {  -16,   5,   2}, 
	{  -16,   5,   2}, {  -16,   5,   2}, {  -16,   5,   2}, {  -16,   5,   2}, 
	{  -16,   5,   2}, {  -16,   5,   2}, {  -16,   5,   2}, {  -16,   5,   2}, 
	{   48,   5,   1}, {   48,   5,   1}, {   48,   5,   1}, {   48,   5,   1}, 
	{   48,   5,   1}, {   48,   5,   1}, {   48,   5,   1}, {   48,   5,   1}, 
	{   48,   5,   1}, {   48,   5,   1}, {   48,   5,   1}, {   48,   5,   1}, 
	{   48,   5,   1}, {   48,   5,   1}, {   48,   5,   1}, {   48,   5,   1}, 
	{  -48,   5,   1}, {  -48,   5,   1}, {  -48,   5,   1}, {  -48,   5,   1}, 
	{  -48,   5,   1}, {  -48,   5,   1}, {  -48,   5,   1}, {  -48,   5,   1}, 
	{  -48,   5,   1}, {  -48,   5,   1}, {  -48,   5,   1}, {  -48,   5,   1}, 
	{  -48,   5,   1}, {  -48,   5,   1}, {  -48,   5,   1}, {  -48,   5,   1}, 
	{   64,   5,   1}, {   64,   5,   1}, {   64,   5,   1}, {   64,   5,   1}, 
	{   64,   5,   1}, {   64,   5,   1}, {   64,   5,   1}, {   64,   5,   1}, 
	{   64,   5,   1}, {   64,   5,   1}, {   64,   5,   1}, {   64,   5,   1}, 
	{   64,   5,   1}, {   64,   5,   1}, {   64,   5,   1}, {   64,   5,   1}, 
	{  -64,   5,   1}, {  -64,   5,   1}, {  -64,   5,   1}, {  -64,   5,   1}, 
	{  -64,   5,   1}, {  -64,   5,   1}, {  -64,   5,   1}, {  -64,   5,   1}, 
	{  -64,   5,   1}, {  -64,   5,   1}, {  -64,   5,   1}, {  -64,   5,   1}, 
	{  -64,   5,   1}, {  -64,   5,   1}, {  -64,   5,   1}, {  -64,   5,   1}, 
	{   16,   6,   3}, {   16,   6,   3}, {   16,   6,   3}, {   16,   6,   3}, 
	{   16,   6,   3}, {   16,   6,   3}, {   16,   6,   3}, {   16,   6,   3}, 
	{  -16,   6,   3}, {  -16,   6,   3}, {  -16,   6,   3}, {  -16,   6,   3}, 
	{  -16,   6,   3}, {  -16,   6,   3}, {  -16,   6,   3}, {  -16,   6,   3}, 
	{   32,   6,   2}, {   32,   6,   2}, {   32,   6,   2}, {   32,   6,   2}, 
	{   32,   6,   2}, {   32,   6,   2}, {   32,   6,   2}, {   32,   6,   2}, 
	{  -32,   6,   2}, {  -32,   6,   2}, {  -32,   6,   2}, {  -32,   6,   2}, 
	{  -32,   6,   2}, {  -32,   6,   2}, {  -32,   6,   2}, {  -32,   6,   2}, 
	{   80,   6,   1}, {   80,   6,   1}, {   80,   6,   1}, {   80,   6,   1}, 
	{   80,   6,   1}, {   80,   6,   1}, {   80,   6,   1}, {   80,   6,   1}, 
	{  -80,   6,   1}, {  -80,   6,   1}, {  -80,   6,   1}, {  -80,   6,   1}, 
	{  -80,   6,   1}, {  -80,   6,   1}, {  -80,   6,   1}, {  -80,   6,   1}, 
	{   96,   6,   1}, {   96,   6,   1}, {   96,   6,   1}, {   96,   6,   1}, 
	{   96,   6,   1}, {   96,   6,   1}, {   96,   6,   1}, {   96,   6,   1}, 
	{  -96,   6,   1}, {  -96,   6,   1}, {  -96,   6,   1}, {  -96,   6,   1}, 
	{  -96,   6,   1}, {  -96,   6,   1}, {  -96,   6,   1}, {  -96,   6,   1}, 
	{   16,   7,   4}, {   16,   7,   4}, {   16,   7,   4}, {   16,   7,   4}, 
	{  -16,   7,   4}, {  -16,   7,   4}, {  -16,   7,   4}, {  -16,   7,   4}, 
	{   16,   7,   5}, {   16,   7,   5}, {   16,   7,   5}, {   16,   7,   5}, 
	{  -16,   7,   5}, {  -16,   7,   5}, {  -16,   7,   5}, {  -16,   7,   5}, 
	{  112,   7,   1}, {  112,   7,   1}, {  112,   7,   1}, {  112,   7,   1}, 
	{ -112,   7,   1}, { -112,   7,   1}, { -112,   7,   1}, { -112,   7,   1}, 
	{  128,   7,   1}, {  128,   7,   1}, {  128,   7,   1}, {  128,   7,   1}, 
	{ -128,   7,   1}, { -128,   7,   1}, { -128,   7,   1}, { -128,   7,   1}, 
	{   16,   8,   6}, {   16,   8,   6}, {  -16,   8,   6}, {  -16,   8,   6}, 
	{   16,   8,   7}, {   16,   8,   7}, {  -16,   8,   7}, {  -16,   8,   7}, 
	{   32,   8,   3}, {   32,   8,   3}, {  -32,   8,   3}, {  -32,   8,   3}, 
	{   48,   8,   2}, {   48,   8,   2}, {  -48,   8,   2}, {  -48,   8,   2}, 
	{   64,   8,   2}, {   64,   8,   2}, {  -64,   8,   2}, {  -64,   8,   2}, 
	{  144,   8,   1}, {  144,   8,   1}, { -144,   8,   1}, { -144,   8,   1}, 
	{  160,   8,   1}, {  160,   8,   1}, { -160,   8,   1}, { -160,   8,   1}, 
	{  176,   8,   1}, {  176,   8,   1}, { -176,   8,   1}, { -176,   8,   1}, 
	{   16,   9,   8}, {  -16,   9,   8}, {   16,   9,   9}, {  -16,   9,   9}, 
	{   16,   9,  10}, {  -16,   9,  10}, {   16,   9,  11}, {  -16,   9,  11}, 
	{   32,   9,   4}, {  -32,   9,   4}, {   32,   9,   5}, {  -32,   9,   5}, 
	{   48,   9,   3}, {  -48,   9,   3}, {   80,   9,   2}, {  -80,   9,   2}, 
	{   96,   9,   2}, {  -96,   9,   2}, {  112,   9,   2}, { -112,   9,   2}, 
	{  192,   9,   1}, { -192,   9,   1}, {  208,   9,   1}, { -208,   9,   1}, 
	{  224,   9,   1}, { -224,   9,   1}, {  240,   9,   1}, { -240,   9,   1}, 
	{  256,   9,   1}, { -256,   9,   1}, {  272,   9,   1}, { -272,   9,   1}, 
	{  512,  -1,   0}, {  514,  -1,   0}, {  516,  -1,   0}, {  518,  -1,   0}, 
	{  520,  -1,   0}, {  522,  -1,   0}, {  524,  -1,   0}, {  526,  -1,   0}, 
	{  528,  -1,   0}, {  530,  -1,   0}, {  532,  -1,   0}, {  534,  -1,   0}, 
	{  536,  -1,   0}, {  538,  -1,   0}, {  540,  -1,   0}, {  542,  -1,   0}, 
	{  544,  -2,   0}, {  548,  -2,   0}, {  552,  -2,   0}, {  556,  -2,   0}, 
	{  560,  -3,   0}, {  568,  -3,   0}, {  576,  -4,   0}, {  592,  -4,   0}, 
	{  608,  -4,   0}, {  624,  -4,   0}, {  640,  -4,   0}, {  656,  -4,   0}, 
	{  672,  -7,   0}, {  800,  -7,   0}, {  928,  -7,   0}, { 1056,  -7,   0}, 
	{   16,   1,  12}, {  -16,   1,  12}, {   16,   1,  13}, {  -16,   1,  13}, 
	{   16,   1,  14}, {  -16,   1,  14}, {   16,   1,  15}, {  -16,   1,  15}, 
	{   32,   1,   6}, {  -32,   1,   6}, {   32,   1,   7}, {  -32,   1,   7}, 
	{   48,   1,   4}, {  -48,   1,   4}, {   48,   1,   5}, {  -48,   1,   5}, 
	{   64,   1,   3}, {  -64,   1,   3}, {   80,   1,   3}, {  -80,   1,   3}, 
	{  128,   1,   2}, { -128,   1,   2}, {  288,   1,   1}, { -288,   1,   1}, 
	{  304,   1,   1}, { -304,   1,   1}, {  320,   1,   1}, { -320,   1,   1}, 
	{  336,   1,   1}, { -336,   1,   1}, {  352,   1,   1}, { -352,   1,   1}, 
	{   48,   2,   6}, {  -48,   2,   6}, {   64,   2,   4}, {  -64,   2,   4}, 
	{   80,   2,   4}, {  -80,   2,   4}, {   96,   2,   3}, {  -96,   2,   3}, 
	{  144,   2,   2}, { -144,   2,   2}, {  160,   2,   2}, { -160,   2,   2}, 
	{  176,   2,   2}, { -176,   2,   2}, {    0,   2,   1}, {    0,   2,   2}, 
	{   48,   3,   7}, {  -48,   3,   7}, {   64,   3,   5}, {  -64,   3,   5}, 
	{   96,   3,   4}, {  -96,   3,   4}, {  192,   3,   2}, { -192,   3,   2}, 
	{  208,   3,   2}, { -208,   3,   2}, {  224,   3,   2}, { -224,   3,   2}, 
	{    0,   3,   3}, {    0,   3,   4}, {    0,   3,   5}, {    0,   3,   6}, 
	{   32,   4,   8}, {  -32,   4,   8}, {   32,   4,   9}, {  -32,   4,   9}, 
	{   32,   4,  10}, {  -32,   4,  10}, {   32,   4,  11}, {  -32,   4,  11}, 
	{   48,   4,   8}, {  -48,   4,   8}, {   48,   4,   9}, {  -48,   4,   9}, 
	{   80,   4,   5}, {  -80,   4,   5}, {  112,   4,   4}, { -112,   4,   4}, 
	{  112,   4,   3}, { -112,   4,   3}, {  128,   4,   3}, { -128,   4,   3}, 
	{  144,   4,   3}, { -144,   4,   3}, {  160,   4,   3}, { -160,   4,   3}, 
	{  176,   4,   3}, { -176,   4,   3}, {  240,   4,   2}, { -240,   4,   2}, 
	{  256,   4,   2}, { -256,   4,   2}, {  272,   4,   2}, { -272,   4,   2}, 
	{    0,   4,   1}, {    0,   4,   2}, {    0,   4,   3}, {    0,   4,   4}, 
	{    0,   4,   5}, {    0,   4,   6}, {    0,   4,   7}, {    0,   4,   8}, 
	{    0,   4,   9}, {    0,   4,  10}, {    0,   4,  11}, {    0,   4,  12}, 
	{    0,   4,  13}, {    0,   4,  14}, {    0,   4,  15}, {    0,   4,  16}, 
	{    0,   4,  17}, {    0,   4,  18}, {    0,   4,  19}, {    0,   4,  20}, 
	{    0,   4,  21}, {    0,   4,  22}, {    0,   4,  23}, {    0,   4,  24}, 
	{    0,   4,  25}, {    0,   4,  26}, {    0,   4,  27}, {    0,   4,  28}, 
	{    0,   4,  29}, {    0,   4,  30}, {    0,   4,  31}, {    0,   4,  32}, 
	{    0,   4,  33}, {    0,   4,  34}, {    0,   4,  35}, {    0,   4,  36}, 
	{    0,   4,  37}, {    0,   4,  38}, {    0,   4,  39}, {    0,   4,  40}, 
	{    0,   4,  41}, {    0,   4,  42}, {    0,   4,  43}, {    0,   4,  44}, 
	{    0,   4,  45}, {    0,   4,  46}, {    0,   4,  47}, {    0,   4,  48}, 
	{    0,   4,  49}, {    0,   4,  50}, {    0,   4,  51}, {    0,   4,  52}, 
	{    0,   4,  53}, {    0,   4,  54}, {    0,   4,  55}, {    0,   4,  56}, 
	{    0,   4,  57}, {    0,   4,  58}, {    0,   4,  59}, {    0,   4,  60}, 
	{    0,   4,  61}, {    0,   4,  62}, {    0,   4,  63}, {    0,   4,  64}, 
	{    0,   6,   1}, {    0,   6,   1}, {   16,   7,   1}, {  -16,   7,   1}, 
	{   32,   7,   1}, {  -32,   7,   1}, {   48,   7,   1}, {  -48,   7,   1}, 
	{   64,   7,   1}, {  -64,   7,   1}, {   80,   7,   1}, {  -80,   7,   1}, 
	{   96,   7,   1}, {  -96,   7,   1}, {  112,   7,   1}, { -112,   7,   1}, 
	{  128,   7,   1}, { -128,   7,   1}, {  144,   7,   1}, { -144,   7,   1}, 
	{  160,   7,   1}, { -160,   7,   1}, {  176,   7,   1}, { -176,   7,   1}, 
	{  192,   7,   1}, { -192,   7,   1}, {  208,   7,   1}, { -208,   7,   1}, 
	{  224,   7,   1}, { -224,   7,   1}, {  240,   7,   1}, { -240,   7,   1}, 
	{  256,   7,   1}, { -256,   7,   1}, {  272,   7,   1}, { -272,   7,   1}, 
	{  288,   7,   1}, { -288,   7,   1}, {  304,   7,   1}, { -304,   7,   1}, 
	{  320,   7,   1}, { -320,   7,   1}, {  336,   7,   1}, { -336,   7,   1}, 
	{  352,   7,   1}, { -352,   7,   1}, {  368,   7,   1}, { -368,   7,   1}, 
	{  384,   7,   1}, { -384,   7,   1}, {  400,   7,   1}, { -400,   7,   1}, 
	{  416,   7,   1}, { -416,   7,   1}, {  432,   7,   1}, { -432,   7,   1}, 
	{  448,   7,   1}, { -448,   7,   1}, {  464,   7,   1}, { -464,   7,   1}, 
	{  480,   7,   1}, { -480,   7,   1}, {  496,   7,   1}, { -496,   7,   1}, 
	{  512,   7,   1}, { -512,   7,   1}, {  528,   7,   1}, { -528,   7,   1}, 
	{  544,   7,   1}, { -544,   7,   1}, {  560,   7,   1}, { -560,   7,   1}, 
	{  576,   7,   1}, { -576,   7,   1}, {  592,   7,   1}, { -592,   7,   1}, 
	{  608,   7,   1}, { -608,   7,   1}, {  624,   7,   1}, { -624,   7,   1}, 
	{  640,   7,   1}, { -640,   7,   1}, {  656,   7,   1}, { -656,   7,   1}, 
	{  672,   7,   1}, { -672,   7,   1}, {  688,   7,   1}, { -688,   7,   1}, 
	{  704,   7,   1}, { -704,   7,   1}, {  720,   7,   1}, { -720,   7,   1}, 
	{  736,   7,   1}, { -736,   7,   1}, {  752,   7,   1}, { -752,   7,   1}, 
	{  768,   7,   1}, { -768,   7,   1}, {  784,   7,   1}, { -784,   7,   1}, 
	{  800,   7,   1}, { -800,   7,   1}, {  816,   7,   1}, { -816,   7,   1}, 
	{  832,   7,   1}, { -832,   7,   1}, {  848,   7,   1}, { -848,   7,   1}, 
	{  864,   7,   1}, { -864,   7,   1}, {  880,   7,   1}, { -880,   7,   1}, 
	{  896,   7,   1}, { -896,   7,   1}, {  912,   7,   1}, { -912,   7,   1}, 
	{  928,   7,   1}, { -928,   7,   1}, {  944,   7,   1}, { -944,   7,   1}, 
	{  960,   7,   1}, { -960,   7,   1}, {  976,   7,   1}, { -976,   7,   1}, 
	{  992,   7,   1}, { -992,   7,   1}, { 1008,   7,   1}, {-1008,   7,   1}, 
	{ 1024,   7,   1}, {-1024,   7,   1}, { 1040,   7,   1}, {-1040,   7,   1}, 
	{ 1056,   7,   1}, {-1056,   7,   1}, { 1072,   7,   1}, {-1072,   7,   1}, 
	{ 1088,   7,   1}, {-1088,   7,   1}, { 1104,   7,   1}, {-1104,   7,   1}, 
	{ 1120,   7,   1}, {-1120,   7,   1}, { 1136,   7,   1}, {-1136,   7,   1}, 
	{ 1152,   7,   1}, {-1152,   7,   1}, { 1168,   7,   1}, {-1168,   7,   1}, 
	{ 1184,   7,   1}, {-1184,   7,   1}, { 1200,   7,   1}, {-1200,   7,   1}, 
	{ 1216,   7,   1}, {-1216,   7,   1}, { 1232,   7,   1}, {-1232,   7,   1}, 
	{ 1248,   7,   1}, {-1248,   7,   1}, { 1264,   7,   1}, {-1264,   7,   1}, 
	{ 1280,   7,   1}, {-1280,   7,   1}, { 1296,   7,   1}, {-1296,   7,   1}, 
	{ 1312,   7,   1}, {-1312,   7,   1}, { 1328,   7,   1}, {-1328,   7,   1}, 
	{ 1344,   7,   1}, {-1344,   7,   1}, { 1360,   7,   1}, {-1360,   7,   1}, 
	{ 1376,   7,   1}, {-1376,   7,   1}, { 1392,   7,   1}, {-1392,   7,   1}, 
	{ 1408,   7,   1}, {-1408,   7,   1}, { 1424,   7,   1}, {-1424,   7,   1}, 
	{ 1440,   7,   1}, {-1440,   7,   1}, { 1456,   7,   1}, {-1456,   7,   1}, 
	{ 1472,   7,   1}, {-1472,   7,   1}, { 1488,   7,   1}, {-1488,   7,   1}, 
	{ 1504,   7,   1}, {-1504,   7,   1}, { 1520,   7,   1}, {-1520,   7,   1}, 
	{ 1536,   7,   1}, {-1536,   7,   1}, { 1552,   7,   1}, {-1552,   7,   1}, 
	{ 1568,   7,   1}, {-1568,   7,   1}, { 1584,   7,   1}, {-1584,   7,   1}, 
	{ 1600,   7,   1}, {-1600,   7,   1}, { 1616,   7,   1}, {-1616,   7,   1}, 
	{ 1632,   7,   1}, {-1632,   7,   1}, { 1648,   7,   1}, {-1648,   7,   1}, 
	{ 1664,   7,   1}, {-1664,   7,   1}, { 1680,   7,   1}, {-1680,   7,   1}, 
	{ 1696,   7,   1}, {-1696,   7,   1}, { 1712,   7,   1}, {-1712,   7,   1}, 
	{ 1728,   7,   1}, {-1728,   7,   1}, { 1744,   7,   1}, {-1744,   7,   1}, 
	{ 1760,   7,   1}, {-1760,   7,   1}, { 1776,   7,   1}, {-1776,   7,   1}, 
	{ 1792,   7,   1}, {-1792,   7,   1}, { 1808,   7,   1}, {-1808,   7,   1}, 
	{ 1824,   7,   1}, {-1824,   7,   1}, { 1840,   7,   1}, {-1840,   7,   1}, 
	{ 1856,   7,   1}, {-1856,   7,   1}, { 1872,   7,   1}, {-1872,   7,   1}, 
	{ 1888,   7,   1}, {-1888,   7,   1}, { 1904,   7,   1}, {-1904,   7,   1}, 
	{ 1920,   7,   1}, {-1920,   7,   1}, { 1936,   7,   1}, {-1936,   7,   1}, 
	{ 1952,   7,   1}, {-1952,   7,   1}, { 1968,   7,   1}, {-1968,   7,   1}, 
	{ 1984,   7,   1}, {-1984,   7,   1}, { 2000,   7,   1}, {-2000,   7,   1}, 
	{ 2016,   7,   1}, {-2016,   7,   1}, { 2032,   7,   1}, {-2032,   7,   1}, 
	{ 2048,   7,   1}, {-2048,   7,   1}, { 2064,   7,   1}, {-2064,   7,   1}, 
	{ 2080,   7,   1}, {-2080,   7,   1}, { 2096,   7,   1}, {-2096,   7,   1}, 
	{ 2112,   7,   1}, {-2112,   7,   1}, { 2128,   7,   1}, {-2128,   7,   1}, 
	{ 2144,   7,   1}, {-2144,   7,   1}, { 2160,   7,   1}, {-2160,   7,   1}, 
	{ 2176,   7,   1}, {-2176,   7,   1}, { 2192,   7,   1}, {-2192,   7,   1}, 
	{ 2208,   7,   1}, {-2208,   7,   1}, { 2224,   7,   1}, {-2224,   7,   1}, 
	{ 2240,   7,   1}, {-2240,   7,   1}, { 2256,   7,   1}, {-2256,   7,   1}, 
	{ 2272,   7,   1}, {-2272,   7,   1}, { 2288,   7,   1}, {-2288,   7,   1}, 
	{ 2304,   7,   1}, {-2304,   7,   1}, { 2320,   7,   1}, {-2320,   7,   1}, 
	{ 2336,   7,   1}, {-2336,   7,   1}, { 2352,   7,   1}, {-2352,   7,   1}, 
	{ 2368,   7,   1}, {-2368,   7,   1}, { 2384,   7,   1}, {-2384,   7,   1}, 
	{ 2400,   7,   1}, {-2400,   7,   1}, { 2416,   7,   1}, {-2416,   7,   1}, 
	{ 2432,   7,   1}, {-2432,   7,   1}, { 2448,   7,   1}, {-2448,   7,   1}, 
	{ 2464,   7,   1}, {-2464,   7,   1}, { 2480,   7,   1}, {-2480,   7,   1}, 
	{ 2496,   7,   1}, {-2496,   7,   1}, { 2512,   7,   1}, {-2512,   7,   1}, 
	{ 2528,   7,   1}, {-2528,   7,   1}, { 2544,   7,   1}, {-2544,   7,   1}, 
	{ 2560,   7,   1}, {-2560,   7,   1}, { 2576,   7,   1}, {-2576,   7,   1}, 
	{ 2592,   7,   1}, {-2592,   7,   1}, { 2608,   7,   1}, {-2608,   7,   1}, 
	{ 2624,   7,   1}, {-2624,   7,   1}, { 2640,   7,   1}, {-2640,   7,   1}, 
	{ 2656,   7,   1}, {-2656,   7,   1}, { 2672,   7,   1}, {-2672,   7,   1}, 
	{ 2688,   7,   1}, {-2688,   7,   1}, { 2704,   7,   1}, {-2704,   7,   1}, 
	{ 2720,   7,   1}, {-2720,   7,   1}, { 2736,   7,   1}, {-2736,   7,   1}, 
	{ 2752,   7,   1}, {-2752,   7,   1}, { 2768,   7,   1}, {-2768,   7,   1}, 
	{ 2784,   7,   1}, {-2784,   7,   1}, { 2800,   7,   1}, {-2800,   7,   1}, 
	{ 2816,   7,   1}, {-2816,   7,   1}, { 2832,   7,   1}, {-2832,   7,   1}, 
	{ 2848,   7,   1}, {-2848,   7,   1}, { 2864,   7,   1}, {-2864,   7,   1}, 
	{ 2880,   7,   1}, {-2880,   7,   1}, { 2896,   7,   1}, {-2896,   7,   1}, 
	{ 2912,   7,   1}, {-2912,   7,   1}, { 2928,   7,   1}, {-2928,   7,   1}, 
	{ 2944,   7,   1}, {-2944,   7,   1}, { 2960,   7,   1}, {-2960,   7,   1}, 
	{ 2976,   7,   1}, {-2976,   7,   1}, { 2992,   7,   1}, {-2992,   7,   1}, 
	{ 3008,   7,   1}, {-3008,   7,   1}, { 3024,   7,   1}, {-3024,   7,   1}, 
	{ 3040,   7,   1}, {-3040,   7,   1}, { 3056,   7,   1}, {-3056,   7,   1}, 
	{ 3072,   7,   1}, {-3072,   7,   1}, { 3088,   7,   1}, {-3088,   7,   1}, 
	{ 3104,   7,   1}, {-3104,   7,   1}, { 3120,   7,   1}, {-3120,   7,   1}, 
	{ 3136,   7,   1}, {-3136,   7,   1}, { 3152,   7,   1}, {-3152,   7,   1}, 
	{ 3168,   7,   1}, {-3168,   7,   1}, { 3184,   7,   1}, {-3184,   7,   1}, 
	{ 3200,   7,   1}, {-3200,   7,   1}, { 3216,   7,   1}, {-3216,   7,   1}, 
	{ 3232,   7,   1}, {-3232,   7,   1}, { 3248,   7,   1}, {-3248,   7,   1}, 
	{ 3264,   7,   1}, {-3264,   7,   1}, { 3280,   7,   1}, {-3280,   7,   1}, 
	{ 3296,   7,   1}, {-3296,   7,   1}, { 3312,   7,   1}, {-3312,   7,   1}, 
	{ 3328,   7,   1}, {-3328,   7,   1}, { 3344,   7,   1}, {-3344,   7,   1}, 
	{ 3360,   7,   1}, {-3360,   7,   1}, { 3376,   7,   1}, {-3376,   7,   1}, 
	{ 3392,   7,   1}, {-3392,   7,   1}, { 3408,   7,   1}, {-3408,   7,   1}, 
	{ 3424,   7,   1}, {-3424,   7,   1}, { 3440,   7,   1}, {-3440,   7,   1}, 
	{ 3456,   7,   1}, {-3456,   7,   1}, { 3472,   7,   1}, {-3472,   7,   1}, 
	{ 3488,   7,   1}, {-3488,   7,   1}, { 3504,   7,   1}, {-3504,   7,   1}, 
	{ 3520,   7,   1}, {-3520,   7,   1}, { 3536,   7,   1}, {-3536,   7,   1}, 
	{ 3552,   7,   1}, {-3552,   7,   1}, { 3568,   7,   1}, {-3568,   7,   1}, 
	{ 3584,   7,   1}, {-3584,   7,   1}, { 3600,   7,   1}, {-3600,   7,   1}, 
	{ 3616,   7,   1}, {-3616,   7,   1}, { 3632,   7,   1}, {-3632,   7,   1}, 
	{ 3648,   7,   1}, {-3648,   7,   1}, { 3664,   7,   1}, {-3664,   7,   1}, 
	{ 3680,   7,   1}, {-3680,   7,   1}, { 3696,   7,   1}, {-3696,   7,   1}, 
	{ 3712,   7,   1}, {-3712,   7,   1}, { 3728,   7,   1}, {-3728,   7,   1}, 
	{ 3744,   7,   1}, {-3744,   7,   1}, { 3760,   7,   1}, {-3760,   7,   1}, 
	{ 3776,   7,   1}, {-3776,   7,   1}, { 3792,   7,   1}, {-3792,   7,   1}, 
	{ 3808,   7,   1}, {-3808,   7,   1}, { 3824,   7,   1}, {-3824,   7,   1}, 
	{ 3840,   7,   1}, {-3840,   7,   1}, { 3856,   7,   1}, {-3856,   7,   1}, 
	{ 3872,   7,   1}, {-3872,   7,   1}, { 3888,   7,   1}, {-3888,   7,   1}, 
	{ 3904,   7,   1}, {-3904,   7,   1}, { 3920,   7,   1}, {-3920,   7,   1}, 
	{ 3936,   7,   1}, {-3936,   7,   1}, { 3952,   7,   1}, {-3952,   7,   1}, 
	{ 3968,   7,   1}, {-3968,   7,   1}, { 3984,   7,   1}, {-3984,   7,   1}, 
	{ 4000,   7,   1}, {-4000,   7,   1}, { 4016,   7,   1}, {-4016,   7,   1}, 
	{ 4032,   7,   1}, {-4032,   7,   1}, { 4048,   7,   1}, {-4048,   7,   1}, 
	{ 4064,   7,   1}, {-4064,   7,   1}, { 4080,   7,   1}, {-4080,   7,   1}, 
};


static const uint8_t dv_zigzag[2][64] = {
	 0,  1,  8, 16,  9,  2,  3, 10, 
	17, 24, 32, 25, 18, 11,  4,  5, 
	12, 19, 26, 33, 40, 48, 41, 34, 
	27, 20, 13,  6,  7, 14, 21, 28, 
	35, 42, 49, 56, 57, 50, 43, 36, 
	29, 22, 15, 23, 30, 37, 44, 51, 
	58, 59, 52, 45, 38, 31, 39, 46, 
	53, 60, 61, 54, 47, 55, 62, 63, 

	 0, 32,  1, 33,  8, 40,  2, 34,
	 9, 41, 16, 48, 24, 56, 17, 49,
	10, 42,  3, 35,  4, 36, 11, 43,
	18, 50, 25, 57, 26, 58, 19, 51,
	12, 44,  5, 37,  6, 38, 13, 45,
	20, 52, 27, 59, 28, 60, 21, 53,
	14, 46,  7, 39, 15, 47, 22, 54,
	29, 61, 30, 62, 23, 55, 31, 63,
};


typedef struct BlockInfo {
    const uint8_t *scan_table;
    uint8_t pos; /* position in block */
    uint8_t dct_mode;
    uint8_t partial_bit_count;
    uint16_t partial_bit_buffer;
    int shift_offset;
} BlockInfo;


/* block size in bits */
static const uint16_t block_sizes[6] = {
    112, 112, 112, 112, 80, 80
};


static __inline int get_bits_left(GetBitContext *s)
{
    return s->size_in_bits - get_bits_count(s);
}


static __inline int get_bits_size(GetBitContext *s)
{
    return s->size_in_bits;
}


static __inline int put_bits_left(PutBitContext* s)
{
    return (int)(s->buf_end - s->buf) * 8 - put_bits_count(s);
}


/* decode ac coefs */
static void dv_decode_ac(GetBitContext *gb, BlockInfo *mb, DCTELEM *block)
{
    int last_index = get_bits_size(gb);
    const uint8_t *scan_table = mb->scan_table;
    int pos = mb->pos;
    int partial_bit_count = mb->partial_bit_count;
    int level, pos1, run, vlc_len, index;

    OPEN_READER(re, gb);
    UPDATE_CACHE(re, gb);

    /* if we must parse a partial vlc, we do it here */
    if (partial_bit_count > 0) {
        re_cache = ((unsigned)re_cache >> partial_bit_count) |
                   (mb->partial_bit_buffer << (sizeof(re_cache)*8 - partial_bit_count));
        re_index -= partial_bit_count;
        mb->partial_bit_count = 0;
    }

    /* get the AC coefficients until last_index is reached */
    for(;;) {
        /* our own optimized GET_RL_VLC */
        index = NEG_USR32(re_cache, TEX_VLC_BITS);
        vlc_len = dv_rl_vlc[index].len;
        if (vlc_len < 0) {
            index = NEG_USR32((unsigned)re_cache << TEX_VLC_BITS, -vlc_len) + dv_rl_vlc[index].level;
            vlc_len = TEX_VLC_BITS - vlc_len;
        }
        level = dv_rl_vlc[index].level;
        run = dv_rl_vlc[index].run;

        /* gotta check if we're still within gb boundaries */
        if (re_index + vlc_len > last_index) {
            /* should be < 16 bits otherwise a codeword could have been parsed */
            mb->partial_bit_count = last_index - re_index;
            mb->partial_bit_buffer = NEG_USR32(re_cache, mb->partial_bit_count);
            re_index = last_index;
            break;
        }
        re_index += vlc_len;

        pos += run;
        if (pos >= 64)
            break;

        pos1 = scan_table[pos];
        block[pos1] = level;

        UPDATE_CACHE(re, gb);
    }
    CLOSE_READER(re, gb);
    mb->pos = pos;
}


static __inline void bit_copy(PutBitContext *pb, GetBitContext *gb)
{
    int bits_left = get_bits_left(gb);
    while (bits_left >= MIN_CACHE_BITS) {
        put_bits(pb, MIN_CACHE_BITS, get_bits(gb, MIN_CACHE_BITS));
        bits_left -= MIN_CACHE_BITS;
    }
    if (bits_left > 0) {
        put_bits(pb, bits_left, get_bits(gb, bits_left));
    }
}


int decode_videosegment_ffmpeg( int16_t *QTMB, uint32_t *quantization_number, uint32_t *DCT_mode, uint32_t *class_number, 
						   uint8_t *DIF_block, uint32_t DIF_sequence_number, uint32_t DIF_block_number )
{
	int n = 0;
    int quant, dc, dct_mode, class1, j;
    int mb_index, last_index;
    DCTELEM *block, *block1;
    uint8_t *buf_ptr;
    PutBitContext pb, vs_pb;
    GetBitContext gb;

    BlockInfo mb_data[5 * 6], *mb, *mb1;

    uint8_t mb_bit_buffer[80 + 4]; /* allow some slack */
    uint8_t vs_bit_buffer[5 * 80 + 4]; /* allow some slack */

    memset(QTMB, 0, 30*128);

    /* pass 1 : read DC and AC coefficients in blocks */
    buf_ptr = DIF_block;
    block1 = QTMB;
    mb1 = mb_data;
    init_put_bits(&vs_pb, vs_bit_buffer, 5 * 80);

    for(mb_index = 0; mb_index < 5; mb_index++, mb1 += 6, block1 += 6 * 64) 
	{
        /* skip header */
        quant = buf_ptr[3] & 0x0f;
        buf_ptr += 4;
        init_put_bits(&pb, mb_bit_buffer, 80);
        mb = mb1;
        block = block1;

		quantization_number[mb_index] = quant;

        for(j = 0; j < 6; j++)
		{
            last_index = block_sizes[j];
            init_get_bits(&gb, buf_ptr, last_index);

            /* get the dc */
            dc = get_sbits(&gb, 9);
            dct_mode = get_bits1(&gb);
            mb->dct_mode = dct_mode;
            mb->scan_table = dv_zigzag[dct_mode];
            class1 = get_bits(&gb, 2);

			DCT_mode[n] = dct_mode;
			class_number[n] = class1;
			n++;
			
			block[0] = dc;
            buf_ptr += last_index >> 3;
            mb->pos = 0;
            mb->partial_bit_count = 0;
            dv_decode_ac(&gb, mb, block);

            /* write the remaining bits  in a new buffer only if the
               block is finished */
            if (mb->pos >= 64)
                bit_copy(&pb, &gb);

            block += 64;
            mb++;
        }

        /* pass 2 : we can do it just after */
        block = block1;
        mb = mb1;
        init_get_bits(&gb, mb_bit_buffer, put_bits_count(&pb));
        flush_put_bits(&pb);
        for(j = 0;j < 6; j++, block += 64, mb++) {
            if (mb->pos < 64 && get_bits_left(&gb) > 0) {
                dv_decode_ac(&gb, mb, block);
                /* if still not finished, no need to parse other blocks */
                if (mb->pos < 64)
                    break;
            }
        }

        /* all blocks are finished, so the extra bytes can be used at
           the video segment level */
        if (j >= 6)
            bit_copy(&vs_pb, &gb);
    }

    // we need a pass over the whole video segment
    block = QTMB;
    mb = mb_data;
    init_get_bits(&gb, vs_bit_buffer, put_bits_count(&vs_pb));
    flush_put_bits(&vs_pb);

    for(mb_index = 0; mb_index < 5; mb_index++) 
	{
        for(j = 0;j < 6; j++) 
		{
            if (mb->pos < 64) 
			{
                dv_decode_ac(&gb, mb, block);
            }

            block += 64;
            mb++;
        }
    }

	return 0;
}

